@@ -13,8 +13,6 @@ from io import StringIO, BytesIO
 from typing import List, Tuple, TextIO, Optional, Set, Dict, cast, Union, \
     Sequence, Any, Iterator
 
-from diskcache import Cache
-
 from benji.blockuidhistory import BlockUidHistory
 from benji.config import Config
 from benji.database import Database, VersionUid, Version, Block, \
@@ -27,6 +25,7 @@ from benji.retentionfilter import RetentionFilter
 from benji.storage.base import InvalidBlockException, BlockNotFoundError
 from benji.storage.factory import StorageFactory
 from benji.utils import notify, BlockHash, PrettyPrint, random_string, InputValidation
+from diskcache import Cache
 
 
 class Benji(ReprMixIn, AbstractContextManager):
@@ -82,7 +81,11 @@ class Benji(ReprMixIn, AbstractContextManager):
         If base_version_uid is given, this is taken as the base, otherwise
         a pure sparse version is created.
         """
-        storage_id = Storage.get_by_name(storage_name).id if storage_name else None
+        if storage_name:
+            new_storage = Storage.get_by_name(storage_name)
+        else:
+            new_storage = Storage.get_by_name(self._default_storage_name)
+
         old_blocks_iter: Optional[Iterator[Block]] = None
         if base_version_uid:
             if not base_version_locking and not Locking.is_version_locked(base_version_uid):
@@ -92,9 +95,8 @@ class Benji(ReprMixIn, AbstractContextManager):
                 raise UsageError('You can only base a new version on a valid old version.')
             if block_size is not None and old_version.block_size != block_size:
                 raise UsageError('You cannot base a new version on an old version with a different block size.')
-            if storage_id is not None and old_version.storage_id != storage_id:
-                raise UsageError('Base version and new version have to be in the same storage.')
-            new_storage_id = old_version.storage_id
+            if old_version.storage != new_storage:
+                raise UsageError(f'Base version and new version have to be in the same storage ({old_version.storage.name} != {new_storage.name}).')
 
             base_version = Version.get_by_uid(base_version_uid)
             old_blocks_iter = base_version.blocks
@@ -106,7 +108,6 @@ class Benji(ReprMixIn, AbstractContextManager):
 
             new_version_block_size = old_version.block_size
         else:
-            new_storage_id = storage_id or Storage.get_by_name(self._default_storage_name).id
             if size is None:
                 raise InternalError('Size needs to be specified if there is no base version.')
             new_size = size
@@ -125,7 +126,7 @@ class Benji(ReprMixIn, AbstractContextManager):
                                      snapshot=snapshot,
                                      size=new_size,
                                      block_size=new_version_block_size,
-                                     storage_id=new_storage_id,
+                                     storage_id=new_storage.id,
                                      status=VersionStatus.incomplete)
             Locking.lock_version(version.uid, reason='Preparing version')
 
@@ -219,17 +220,17 @@ class Benji(ReprMixIn, AbstractContextManager):
                 'Preparing {} of version {} ({:.1f}%)'.format('deep-scrub' if deep_scrub else 'scrub', version.uid,
                                                               (i + 1) / version.blocks_count * 100))
             if not block.uid:
-                logger.debug('{} of block {} (UID {}) skipped (sparse).'.format('Deep-scrub' if deep_scrub else 'Scrub',
-                                                                                block.idx, block.uid))
+                logger.debug('{} of block {} of version {} (UID {}) skipped (sparse).'.format(
+                    'Deep-scrub' if deep_scrub else 'Scrub', block.idx, version.uid, block.uid))
                 continue
             if history and history.seen(version.storage_id, block.uid):
-                logger.debug('{} of block {} (UID {}) skipped (already seen).'.format(
-                    'Deep-scrub' if deep_scrub else 'Scrub', block.idx, block.uid))
+                logger.debug('{} of block {} of version {} (UID {}) skipped (already seen).'.format(
+                    'Deep-scrub' if deep_scrub else 'Scrub', block.idx, version.uid, block.uid))
                 continue
-            # i != 0 makes sure that we always scrub at least one block (the first in this case)
+            # i != 0 ensures that we always scrub at least one block (the first in this case)
             if i != 0 and block_percentage < 100 and random.randint(1, 100) > block_percentage:
-                logger.debug('{} of block {} (UID {}) skipped (percentile is {}).'.format(
-                    'Deep-scrub' if deep_scrub else 'Scrub', block.idx, block.uid, block_percentage))
+                logger.debug('{} of block {} of version {} (UID {}) skipped (percentile is {}).'.format(
+                    'Deep-scrub' if deep_scrub else 'Scrub', block.idx, version.uid, block.uid, block_percentage))
             else:
                 storage.read_block_async(block, metadata_only=(not deep_scrub))
                 read_jobs += 1
@@ -274,8 +275,10 @@ class Benji(ReprMixIn, AbstractContextManager):
                 if isinstance(entry, Exception):
                     # If it really is a data inconsistency mark blocks invalid
                     if isinstance(entry, InvalidBlockException):
-                        logger.error('Block {} (UID {}) is invalid: {}'.format(entry.block.idx, entry.block.uid, entry))
-                        affected_version_uids.extend(Version.set_block_invalid(entry.block.uid))
+                        logger.error('Block {} (UID {}) is invalid: {}{}'.format(
+                            entry.block.idx, entry.block.uid, entry,
+                            f' Caused by: {entry.__cause__}' if entry.__cause__ else ''))
+                        affected_version_uids.extend(Version.set_block_valid(entry.block.uid, False))
                         valid = False
                         continue
                     else:
@@ -288,7 +291,7 @@ class Benji(ReprMixIn, AbstractContextManager):
                 except (KeyError, ValueError) as exception:
                     logger.error('Metadata check failed, block {} (UID {}) is invalid: {}'.format(
                         block.idx, block.uid, exception))
-                    affected_version_uids.extend(Version.set_block_invalid(block.uid))
+                    affected_version_uids.extend(Version.set_block_valid(block.uid, False))
                     valid = False
                     continue
 
@@ -332,6 +335,7 @@ class Benji(ReprMixIn, AbstractContextManager):
             if not version.status.is_deep_scrubbable():
                 raise ScrubbingError('Version {} cannot be deep-scrubbed, it has a status of {}.'.format(
                     version_uid, version.status.name))
+            # This only outputs a warning to let the user know that the version it not valid to begin with.
             if not version.status.is_valid():
                 logger.warn('Version {} has a status of {}.'.format(version_uid, version.status.name))
 
@@ -359,8 +363,10 @@ class Benji(ReprMixIn, AbstractContextManager):
                 if isinstance(entry, Exception):
                     # If it really is a data inconsistency mark blocks invalid
                     if isinstance(entry, InvalidBlockException):
-                        logger.error('Block {} (UID {}) is invalid: {}'.format(entry.block.idx, entry.block.uid, entry))
-                        affected_version_uids.extend(Version.set_block_invalid(entry.block.uid))
+                        logger.error('Block {} (UID {}) is invalid: {}{}'.format(
+                            entry.block.idx, entry.block.uid, entry,
+                            f' Caused by: {entry.__cause__}' if entry.__cause__ else ''))
+                        affected_version_uids.extend(Version.set_block_valid(entry.block.uid, False))
                         valid = False
                         continue
                     else:
@@ -371,35 +377,41 @@ class Benji(ReprMixIn, AbstractContextManager):
                 try:
                     storage.check_block_metadata(block=block, data_length=len(data), metadata=metadata)
                 except (KeyError, ValueError) as exception:
-                    logger.error('Metadata check failed, block {} (UID {}) is invalid: {}'.format(
-                        block.idx, block.uid, exception))
-                    Version.set_block_invalid(block.uid)
+                    logger.error('Metadata check failed, block {} of version {} (UID {}) is invalid: {}'.format(
+                        block.idx, version_uid, block.uid, exception))
+                    Version.set_block_valid(block.uid, False)
                     valid = False
                     continue
 
                 data_checksum = self._block_hash.data_hexdigest(data)
                 if data_checksum != block.checksum:
                     logger.error(
-                        'Checksum mismatch during deep-scrub of block {} (UID {}) (is: {}... should-be: {}...).'.format(
-                            block.idx, block.uid, data_checksum[:16],
+                        'Checksum mismatch during deep-scrub of block {} of version {} (UID {}) (is: {}... should-be: {}...).'.format(
+                            block.idx, version_uid, block.uid, data_checksum[:16],
                             cast(str, block.checksum)[:16]))  # We know that block.checksum is set
-                    affected_version_uids.extend(Version.set_block_invalid(block.uid))
+                    affected_version_uids.extend(Version.set_block_valid(block.uid, False))
                     valid = False
                     continue
 
                 if source:
                     source_data = io.read_sync(block)
                     if source_data != data:
-                        logger.error('Source data has changed for block {} (UID {}) (is: {}... should-be: {}...). '
-                                     'Won\'t set this block to invalid, because the source looks wrong.'.format(
-                                         block.idx, block.uid,
+                        logger.error('Source data has changed for block {} of version {} (UID {}) (is: {}... should-be: {}...). '
+                                     'Will not mark this block as invalid, because the source looks wrong.'.format(
+                                         block.idx, version_uid, block.uid,
                                          self._block_hash.data_hexdigest(source_data)[:16], data_checksum[:16]))
                         valid = False
-                        # We are not setting the block invalid here because
-                        # when the block is there AND the checksum is good,
-                        # then the source is probably invalid.
+                        # We are not setting the block invalid here because when the block exists and the checksum is
+                        # correct then the source is probably invalid.
                         source_mismatch = True
 
+                if not block.valid:
+                    logger.info('Block {} of version {} (UID {}) passed revalidation, marking it as valid.'.format(
+                        block.idx, version_uid, block.uid))
+                    Version.set_block_valid(block.uid, True)
+
+                # Only add the block to the history if it is valid. This ensure that this block will also get flagged
+                # again when deep-scrubbing other versions containing it.
                 if history:
                     history.add(version.storage_id, block.uid)
 
@@ -438,7 +450,7 @@ class Benji(ReprMixIn, AbstractContextManager):
             if version.uid in affected_version_uids:
                 affected_version_uids.remove(version.uid)
             if affected_version_uids:
-                logger.error('Marked the following versions as invalid, too, because of invalid blocks: {}.' \
+                logger.error('Marked the following versions as invalid too because of invalid blocks: {}.' \
                              .format(', '.join(sorted(affected_version_uids))))
 
         Locking.unlock_version(version_uid)
@@ -464,7 +476,7 @@ class Benji(ReprMixIn, AbstractContextManager):
 
         if version_percentage and versions:
             # Will always scrub at least one matching version
-            versions = set(random.sample(versions, max(1, int(len(versions) * version_percentage / 100))))
+            versions = set(random.sample(list(versions), max(1, int(len(versions) * version_percentage / 100))))
         if not versions:
             logger.info('No matching versions found.')
             return [], []
@@ -476,7 +488,7 @@ class Benji(ReprMixIn, AbstractContextManager):
                     version.volume))
                 getattr(self, method)(version.uid, block_percentage=block_percentage, history=history)
             except ScrubbingError as exception:
-                logger.error(exception)
+                logger.error(str(exception))
                 errors.append(version)
             except AlreadyLocked:
                 logger.warning(f'Skipping version {version.uid}, it is locked. ')
@@ -606,7 +618,7 @@ class Benji(ReprMixIn, AbstractContextManager):
                     logger.error('Storage backend read failed: {}'.format(entry))
                     # If it really is a data inconsistency mark blocks invalid
                     if isinstance(entry, (KeyError, ValueError)):
-                        Version.set_block_invalid(block.uid)
+                        Version.set_block_valid(block.uid, False)
                         continue
                     else:
                         raise entry
@@ -621,7 +633,7 @@ class Benji(ReprMixIn, AbstractContextManager):
                     storage.check_block_metadata(block=block, data_length=len(data), metadata=metadata)
                 except (KeyError, ValueError) as exception:
                     logger.error('Metadata check failed, block is invalid: {}'.format(exception))
-                    Version.set_block_invalid(block.uid)
+                    Version.set_block_valid(block.uid, False)
                     continue
 
                 data_checksum = self._block_hash.data_hexdigest(data)
@@ -630,7 +642,7 @@ class Benji(ReprMixIn, AbstractContextManager):
                                  'block.valid: {}). Block restored is invalid.'.format(
                                      block.idx, block.uid, data_checksum[:16],
                                      cast(str, block.checksum)[:16], block.valid))  # We know that block.checksum is set
-                    Version.set_block_invalid(block.uid)
+                    Version.set_block_valid(block.uid, False)
                 else:
                     logger.debug('Restored block {} successfully ({} bytes).'.format(block.idx, block.size))
 
@@ -825,6 +837,7 @@ class Benji(ReprMixIn, AbstractContextManager):
                     raise InputDataError('Source changed in regions outside of ones indicated by the hints.')
             logger.info('Finished sanity check. Checked {} blocks.'.format(read_jobs))
 
+        logger.info(f'Starting backup of {io.sanitized_url} to storage {version.storage.name}, the newly created version is {version.uid}.')
         try:
             storage = StorageFactory.get_by_name(version.storage.name)
             read_jobs = 0
@@ -970,7 +983,7 @@ class Benji(ReprMixIn, AbstractContextManager):
 
         Locking.unlock_version(version.uid)
         notify(self._process_name)
-        logger.info('New version {} created, backup successful.'.format(version.uid))
+        logger.info('Backup successful.'.format(version.uid))
         return version
 
     def cleanup(self, dt: int = 3600, override_lock: bool = False) -> None:
