@@ -32,10 +32,6 @@ from alembic import command as alembic_command
 from alembic.config import Config as alembic_config_Config
 from alembic.runtime.environment import EnvironmentContext
 from alembic.script import ScriptDirectory
-from sqlalchemy import func
-from sqlalchemy.orm import object_session, sessionmaker, scoped_session, aliased
-from sqlalchemy.orm.collections import attribute_mapped_collection
-
 from benji.config import Config
 from benji.exception import InputDataError, InternalError, AlreadyLocked, UsageError, ConfigurationError
 from benji.logging import logger
@@ -43,6 +39,9 @@ from benji.repr import ReprMixIn
 from benji.storage.key import StorageKeyMixIn
 from benji.utils import InputValidation
 from benji.versions import VERSIONS
+from sqlalchemy import func
+from sqlalchemy.orm import object_session, sessionmaker, scoped_session, aliased
+from sqlalchemy.orm.collections import attribute_mapped_collection
 
 
 # SQLite 3 supports checking of foreign keys but it needs to be enabled explicitly!
@@ -59,6 +58,9 @@ class BenjiDateTime(sqlalchemy.types.TypeDecorator):
 
     impl = sqlalchemy.DateTime
 
+    # This is non-cacheable as it may be a relative time specifications and so is dependent on the current time.
+    cache_ok = False
+
     def process_bind_param(self, value: Optional[Union[datetime.datetime, str]], dialect) -> Optional[datetime.datetime]:
         if isinstance(value, datetime.datetime):
             if value.tzinfo is None:
@@ -111,6 +113,8 @@ class VersionStatusType(sqlalchemy.types.TypeDecorator):
 
     impl = sqlalchemy.Integer
 
+    cache_ok = True
+
     def process_bind_param(self, value: Optional[Union[int, str, VersionStatus]], dialect) -> Optional[int]:
         if value is None:
             return None
@@ -161,6 +165,8 @@ class VersionUidType(sqlalchemy.types.TypeDecorator):
 
     impl = sqlalchemy.String(255)
 
+    cache_ok = True
+
     def process_bind_param(self, value: Optional[str], dialect) -> Optional[str]:
         if value is None or isinstance(value, str):
             return value
@@ -178,6 +184,8 @@ class ChecksumType(sqlalchemy.types.TypeDecorator):
 
     impl = sqlalchemy.LargeBinary
 
+    cache_ok = True
+
     def process_bind_param(self, value: Optional[str], dialect) -> Optional[bytes]:
         if value is not None:
             return unhexlify(value)
@@ -471,22 +479,30 @@ class Version(Base, ReprMixIn):
             raise
 
     @classmethod
-    def set_block_invalid(cls, block_uid: BlockUid) -> List[VersionUid]:
+    def set_block_valid(cls, block_uid: BlockUid, valid: bool) -> List[VersionUid]:
         try:
-            affected_version_uids_query = Session.query(sqlalchemy.distinct(
-                Version.uid).label('uid')).join(Block).filter(Block.uid == block_uid)
-            affected_version_uids = [row.uid for row in affected_version_uids_query]
-            assert len(affected_version_uids) > 0
+            # Can't use DISTINCT here as PostgreSQL doesn't support DISTINCT together with FOR UPDATE.
+            affected_version_uids_query = Session.query(cls.uid.label('uid')).join(Block).filter(Block.uid == block_uid)
+            if not valid:
+                affected_version_uids_query = affected_version_uids_query.with_for_update()
+            # Use a set to replace the missing DISTINCT above.
+            affected_version_uids = set([row.uid for row in affected_version_uids_query])
 
-            Session.query(Block).filter(Block.uid == block_uid).update({'valid': False}, synchronize_session=False)
-            Session.commit()
+            Session.query(Block).filter(Block.uid == block_uid).update({'valid': valid}, synchronize_session=False)
 
-            logger.error('Marked block with UID {} as invalid. Affected versions: {}.'.format(
-                block_uid, ', '.join(affected_version_uids)))
+            if len(affected_version_uids) > 0:
+                logger.error('Marked block with UID {} as {} in all affected versions: {}.'.format(
+                    block_uid, 'valid' if valid else 'invalid', ', '.join(affected_version_uids)))
+
+                # We won't mark any versions as valid because they might contain other invalid blocks.
+                if not valid:
+                    Session.query(cls).filter(cls.uid.in_(affected_version_uids)).update(
+                        {'status': VersionStatus.invalid}, synchronize_session=False)
+            else:
+                # Version could have been deleted in the meantime
+                logger.warning('No version was affected by marking block with UID {} as {}.'.format(
+                    block_uid, 'valid' if valid else 'invalid'))
 
-            for version_uid in affected_version_uids:
-                version = Version.get_by_uid(version_uid)
-                version.set(status=VersionStatus.invalid)
             Session.commit()
         except:
             Session.rollback()
